/*******************************************************************
  User space IRQ handling emulator for i386 Linux based PCs
  It uses VM86_PLUS services developed for DOSEMU

  emu_irq.c

  (C) Copyright 1998 by Pavel Pisa 

  This code is distributed under the Gnu General Public Licence. 
  See file COPYING for details.
 *******************************************************************/
 
#include <stdio.h>
#include <sys/io.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include "emu_irq.h"

/*******************************************************************/
/* usage example */

#define LPT_REG_BASE 0x378
#define LPT_IRQ      7

#define DATA_REG     0
#define STATUS_REG   1
#define CONTROL_REG  2

typedef unsigned char byte;

int signal_on = 0;
int interrupt_cnt = 0;

/* signal handler */
void signal_handler(int sig)
{
    printf("SIG INT ...\n");
    signal_on = 1;
}

void my_irq_handler(int intno, void *dev_id, void *regs)
{
    interrupt_cnt++;
};

 
int main(int argc, char **argv)
{ 
    int ret;
    int dir;
    int irc;
    unsigned long par_port_base = LPT_REG_BASE;

    signal(SIGINT, signal_handler);    
    
    ret = ioperm(LPT_REG_BASE, 3, 1);
    if(ret<0) { 
            perror("ioperm"); 
            return 4;
    }

    printf("requesting interrupt handler\n");
    ret = request_emu_irq(LPT_IRQ, my_irq_handler, 0, "motor_test", &par_port_base);
    if(ret < 0) { 
            perror("request_emu_irq");
            return 4;
    }
    
    byte old37A = inb(LPT_REG_BASE + CONTROL_REG);
    outb((old37A & ~0x20) | 0x10, LPT_REG_BASE + CONTROL_REG); // enable IRQ in 8255

    outb(0x01, LPT_REG_BASE);   /* Rotate the motor */
    dir = 1;

    while(!signal_on) {
            usleep(200*1000);
            if ((interrupt_cnt > 20000) && (dir == 1)) {
                    outb(0x02, LPT_REG_BASE); /* Stop the motor */
                    dir = 2;
	    }

            if ((interrupt_cnt > 40000) && (dir != 0)) {
                    outb(0x00, LPT_REG_BASE); /* Stop the motor */
                    dir = 0;
	    }

            irc = inb(0x379);
            printf("\rinterrupt count: %8i %c %c", interrupt_cnt,
	           irc & (1 << 4)? 'A': '-',  irc & (1 << 5)? 'B': '-');
            fflush(stdout);
    }
    printf("\n");

    outb(old37A, LPT_REG_BASE + CONTROL_REG); // disable IRQ in 8255

    printf("unregistering interrupt handler\n");
    release_emu_irq(LPT_IRQ, &par_port_base);

    printf("stopping the motor.\n");
    outb(0, LPT_REG_BASE + DATA_REG);

    printf("releasing printer port.\n");
    ioperm(LPT_REG_BASE, 3, 0);

    printf("interrupt cnt: %i\n", interrupt_cnt);
    printf("bye ...\n");
    
    return 0;
};


